/*  The OVLY_MGR.ASM file is the overlay manager.  When a symbol  	*/
/*  residing in overlay is referenced, the ovelay manager loads   	*/
/*  the overlay code and begins execution.  (This overlay manager 	*/
/*  does not check to see if the overlay is already in internal   	*/
/*  memory, this option will be added later.)  A DMA transfer is  	*/
/*  performed to load in the meory overlay.			  				*/

#include "def21065L.h"  

 
.SEGMENT/DM             dm_data;

/*  The following constants are defined by the linker.  */
/*  These constants contain the word size, live locaton */
/*  and run location of the overlay functions.	        */

.EXTERN _ov_word_run_size_1;
.EXTERN _ov_word_run_size_2;
.EXTERN _ov_word_run_size_3;
.EXTERN _ov_word_live_size_1;
.EXTERN _ov_word_live_size_2;
.EXTERN _ov_word_live_size_3;
.EXTERN _ov_startaddress_1;
.EXTERN _ov_startaddress_2;
.EXTERN _ov_startaddress_3;
.EXTERN _ov_runtimestartaddress_1;
.EXTERN _ov_runtimestartaddress_2;
.EXTERN _ov_runtimestartaddress_3;
/*  Placing the linker constants in an array so the overlay */
/*  manager can use the appropriate constant based on the   */
/*  overlay id.						     					*/

.VAR  liveAddresses[3] = _ov_startaddress_1,        _ov_startaddress_2,			_ov_startaddress_3; 
.VAR  runAddresses[3]  = _ov_runtimestartaddress_1, _ov_runtimestartaddress_2,  _ov_runtimestartaddress_3;
.VAR  runWordSize[3]   = _ov_word_size_run_1,       _ov_word_size_run_2,		_ov_word_size_run_3;
.VAR  liveWordSize[3]  = _ov_word_size_live_1,      _ov_word_size_live_2,		_ov_word_size_live_3;

/*  software stack to temporarily store registers corrupted by overlay manager */
.VAR  ov_stack[10]; 
.VAR  ov_id_loaded = -1;

.ENDSEG;

/************************************************************************/
/*  		Overlay Manager Function									*/

.SEGMENT/PM             pm_code;                                                         

_OverlayManager: 
.GLOBAL _OverlayManager;

/*  _overlayID has been defined as R0.  R0 is set in the PLIT of LDF.	*/
/*  Set up DMA transfer to internal memory through the external port.	*/


/*  Store values of registers used by the overlay manager in to the */
/*  software stack.						    						*/
dm(ov_stack)=i8;
dm(ov_stack+1)=m8;
dm(ov_stack+2)=l8;
dm(ov_stack+3)=r2;

/* Use the overlay id as an index (must subtract one)  	*/
R0=R0-1; /* Overlay ID -1 								*/
m8=R0;   /*  Offset into the arrays containing linker 	*/
         /*  defined overlay constants.               	*/

r2=dm(ov_id_loaded);
r0=r0-r2;
if EQ jump continue;
dm(ov_id_loaded)=m8;

r0=i0;	dm(ov_stack+4)=r0;
r0=m0;	dm(ov_stack+5)=r0;
r0=l0;	dm(ov_stack+6)=r0;

l8=0;
l0=0;

m0=m8; /* Overlay ID - 1 */

/*  Get overlay run and live addresses from memory and use to 	*/
/*  set up the master mode DMA.				      				*/
i8 = runAddresses;
i0 = liveAddresses;

r0=0;			
/* Disable DMA */
dm(DMAC0) = r0;	 

/* Set DMA external pointer to overlay live address */
r0=dm(m0,i0);
dm(EIEP0)=r0;

/* Set DMA internal pointer to overlay run address  */
r0=pm(m8,i8);
dm(IIEP0)=r0;

i0=runWordSize;  /*  Number of words stored in internal memory  */
                 /*  Most likely the word size will be 48 bits  */
                 /*  for instructions.							*/
			
/* Set DMA external modifier */
r0=1;
dm(EMEP0)=r0;

i8=liveWordSize; /*  Number of words stored in external memory  */
                 /*  Most likely the word size will be 32 or16  */
                 /*  bits for external storage.			*/	

/* Set DMA internal modify to 1 */
dm(IMEP0)=r0;
	
/* Set DMA internal count to Overlay run size.	*/
r0=dm(m0,i0);
dm(CEP0)=r0;
 
/* Set DMA external count to Overlay live size.	*/
r0=pm(m8,i8);


/********************* BOOT SPACE OVERLAYS  *********************/
ustat1=dm(SYSCON);
bit set ustat1 0x00000002; 	/* Select Boot Override forcing 8-48 bit external reads */
dm(SYSCON)=ustat1;
r2=0x6;						/* Each 48-bit word is 6 bytes wide in boot space */
r0=r0*r2(SSI);
/****************************************************************/

dm(ECEP0)=r0;
			
/* DMA enabled, instruction word, Master, 48-32 packing - Overriden is BMO set */ 		
r0=0x0221;  
dm(DMAC0)=r0;

/*  Enable DMA interrupt */
bit set mode1 IRPTEN;
bit set imask EP0I;

/*  Restore register values from stack  */
r0=dm(ov_stack+6);	l0=r0;
r0=dm(ov_stack+5);	m0=r0;
r0=dm(ov_stack+4);	i0=r0;

/*  Wait for DMA to complete  */
dma1_wait:		idle;

continue:
r2=dm(ov_stack+3);
l8=dm(ov_stack+2);
i8=r1;
m8=0;
r1=dm(ov_stack+1);
r0=dm(ov_stack);

/*  Flush the cache.  If an instruction in previous overlay  */
/*  had been cached, it may be executed instead of the       */
/*  current overlays instruction.  (If pm transfers align.)  */
flush cache;

/*  Jump to the location of the function to be executed.  */
jump (m8,i8) (db);
i8=r0;
m8=r1;

.ENDSEG;
/*																		*/
/************************************************************************/


